package de.twenty11.skysail.common.ext.osgimonitor;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.osgi.framework.Version;

import de.twenty11.skysail.common.graphs.EdgeProvider;
import de.twenty11.skysail.common.graphs.NodeProvider;

public class BundleDetails implements Serializable {

    private static final long serialVersionUID = -1079406626730388016L;

    public static final String SYMBOLIC_NAME = "symbolicName";

    private String symbolicName;

	private long bundleId;

	private long lastModified;

	private List<ServiceReferenceDetails> registeredServices = new ArrayList<ServiceReferenceDetails>();

	private List<ServiceReferenceDetails> servicesInUse = new ArrayList<ServiceReferenceDetails>();

	private int state;

	private Version version;
    
    public BundleDetails() {
        // needed for javax.persistence
    }

    public String getSymbolicName() {
        return symbolicName;
    }
    
    public void setSymbolicName(String symbolicName) {
        this.symbolicName = symbolicName;
    }

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(symbolicName).append(", ");
        return sb.toString();
    }

	public void setBundleId(long bundleId) {
		this.bundleId = bundleId;
	}
	
	public long getBundleId() {
		return bundleId;
	}

	public void setLastModified(long lastModified) {
		this.lastModified = lastModified;
	}
	
	public long getLastModified() {
		return lastModified;
	}

	public void setRegisteredServices(List<ServiceReferenceDetails> list) {
		this.registeredServices = list;
	}

	public List<ServiceReferenceDetails> getRegisteredServices() {
		return registeredServices;
	}

	public void setServicesInUse(List<ServiceReferenceDetails> list) {
		this.servicesInUse = list;
	}
	
	public List<ServiceReferenceDetails> getServicesInUse() {
		return servicesInUse;
	}

	public void setState(int state) {
		this.state = state;
	}
	
	public int getState() {
		return state;
	}

	public void setVersion(Version version) {
		this.version = version;
	}
	
	public Version getVersion() {
		return version;
	}

    @JsonIgnore
    public NodeProvider asNode() {
        return new NodeProvider() {

            @Override
            public String getType() {
                return BundleDetails.this.getClass().getCanonicalName();
            }

            @Override
            public Map<String, String> getNodeProperties() {
                return Collections.emptyMap();
            }

            @Override
            public String getNodeLabel() {
                return symbolicName + "(" + version.toString() + ")";
            }

            @Override
            public String getNodeId() {
                return Long.toString(bundleId);
            }

            @Override
            public List<EdgeProvider> getEdges() {
                List<EdgeProvider> result = new ArrayList<EdgeProvider>();
                for (final ServiceReferenceDetails srd : BundleDetails.this.registeredServices) {
                    if (srd.getUsingBundles().size() == 0) {
                        continue;
                    }
                    for (final BundleDetails usingBundle : srd.getUsingBundles()) {
                        EdgeProvider edgeProvider = new EdgeProvider() {

                            @Override
                            public int getWeight() {
                                return 1;
                            }

                            @Override
                            public String getTarget() {
                                return Long.toString(usingBundle.getBundleId());
                            }

                            @Override
                            public String getSource() {
                                return BundleDetails.this.asNode().getNodeId();
                            }

                            @Override
                            public Map<String, String> getEdgeProperties() {
                                return Collections.emptyMap();
                            }

                            @Override
                            public String getEdgeLabel() {
                                return srd.getName();
                            }

                            @Override
                            public String getEdgeId() {
                                return srd.getName();
                            }
                        };
                        result.add(edgeProvider);

                    }
                }
                return result;
            }
        };
    }
	

}
